///////////////////////////////////////////////////////////////////////////////
//
// Meltice 2.x source code.
//
///////////////////////////////////////////////////////////////////////////////
// C_INISET.CPP = Config group / INI file SETtings
///////////////////////////////////////////////////////////////////////////////
// $Id: C_INISET.CPP,v 1.21 2001/06/16 16:35:34 ardry Exp $ 
///////////////////////////////////////////////////////////////////////////////

#include	<windows.h>
#include	"u_util.h"
#include	"c_config.h"

static const char cGlobalSection[8] = "Global";
static const char cUnPackSection[8] = "UnPack";
static const char cPackSection[8]   = "Pack";

//Generate user's ini filename.
void GetINIfilename(char *Path,const int PathLen)
{
	char INIfilename[MAX_PATH];
	GetExecDir(Path,PathLen);

	char buf[256]; unsigned long buflen=255;
	if(GetUserName(buf,&buflen)){
		BadNameCheckConv(buf, '_'); //cut probrem character by usrname.
		lstrcpyn(INIfilename,buf,MAX_PATH);
		strncat(INIfilename,".ini",MAX_PATH-strlen(INIfilename));
	}else{
		lstrcpyn(INIfilename,"MELTICE.INI",MAX_PATH);
	}


	strncat(Path,INIfilename,MAX_PATH-strlen(Path));
	return;
}

//Check folder write permission.
// return -> true = INI can write / false = INI cannot write.
static bool ChkIniPermission(void)
{
	char inipath[MAX_PATH];
	GetINIfilename(inipath,MAX_PATH);
	HANDLE hFp;

////ini file write test.
	hFp = CreateFile(inipath,
							GENERIC_READ|GENERIC_READ,
							FILE_SHARE_WRITE,
							NULL,
							OPEN_ALWAYS,
							FILE_ATTRIBUTE_NORMAL,
							NULL);
	if(hFp == INVALID_HANDLE_VALUE )
		return false;		// No INI write permission.
	CloseHandle(hFp);

	lstrcat(inipath,".$$$");
////temp file write test.
	hFp = CreateFile(inipath,
							GENERIC_READ|GENERIC_READ,
							FILE_SHARE_WRITE,
							NULL,
							OPEN_ALWAYS,
							FILE_ATTRIBUTE_NORMAL,
							NULL);
	if(hFp == INVALID_HANDLE_VALUE )
		return false;		// No INI write permission.
	CloseHandle(hFp);
	if(DeleteFile(inipath) == false) //tmp file delete failed!
		return false;

	return true;
}

// load settings from ini file.
void InicfgToMemcfg(void)
{
	//INI path setting...
	char *inipath = new char[MAX_PATH];

	char *buf = new char[MAX_PATH*4];
	GetINIfilename(inipath,MAX_PATH);

	DWORD dwIniAttr = GetFileAttributes(inipath);

	//for FAT and NTFS.
	if( dwIniAttr == 0xFFFFFFFF )
		set_cfg_readonly(false);	// Not Found
	else if( dwIniAttr & FILE_ATTRIBUTE_READONLY )
		set_cfg_readonly(true);		// Read Only (attribute)
	else
		set_cfg_readonly(false);	//Read/Write OK.
	//for NTFS and CD-ROM.
	if( ChkIniPermission() == false )
		set_cfg_readonly(true);	// Read Only (NTFS permission?)

	//DesktopDir...
	char *desktopdir = new char[MAX_PATH];
	GetWinSpecialFolderPath(desktopdir, CSIDL_DESKTOP);
	Path_Add_Backslash(desktopdir);

// [Global] Section.
	//AutoLang(Japanese or US-English switching.), default=true.
	GetPrivateProfileString(cGlobalSection,"AutoLang","1",buf,2,inipath);
	if(*buf=='0')	set_autolang( false);	// AutoLang=0 -> false
	else 			set_autolang(  true);	// AutoLang=1 -> true

	set_dialog_x(GetPrivateProfileInt(cGlobalSection,"Dialog_X",-9999,inipath));
	set_dialog_y(GetPrivateProfileInt(cGlobalSection,"Dialog_Y",-9999,inipath));

	//WriteLog(Processing Log, for debug.), default=false.
	GetPrivateProfileString(cGlobalSection,"WriteLog","0",buf,2,inipath);
	if(*buf=='0')	set_writelog( false);	// WriteLog=0 -> false
	else 			set_writelog(  true);	// WriteLog=1 -> true


// [UnPack] Section.
	//meltdir load...
	GetPrivateProfileString(cUnPackSection,"DestDir",desktopdir,buf,MAX_PATH,inipath);
	// --- vvv ---  '//hogehoge' shortcut convert (may be etcfunc...)
	if( *buf=='/' && *(buf+1)=='/' )
	{	// shortcut, ex... '//desktop' 
		if( *(buf+2) == 'd' )
		{ //desktop shortcut.
			set_destdir(desktopdir, MAX_PATH, 0);
		}else if( *(buf+2) == 'm' )
		{ // '//mydoc' = My Documents shortcut.
			set_destdir(
				GetWinSpecialFolderPath(buf, CSIDL_PERSONAL), 
				MAX_PATH, 0);
			//for Windows 95 without MS-Office 9x/2000
			if(*buf=='\0') set_destdir(desktopdir, MAX_PATH, 0);
		}else if( *(buf+2) == 't' )
		{ // '//temp' = temp shortcut.
			GetTempPath(MAX_PATH,buf);
			GetLongPathName95(buf,buf); //for LFN TempPath on Windows 2000 and XP.
			Path_Add_Backslash(buf);
			set_destdir(buf, MAX_PATH, 0);
		}else if( *(buf+2) == 'e' )
		{ // '//execdir' Meltice.exe path shortcut. 
			GetExecDir(buf,MAX_PATH);
			set_destdir(buf, MAX_PATH, 0);
		}else // etc shortcut = '//desktop'
		{ 
			// '//???' into memory when not reserved word.
			set_destdir(desktopdir, MAX_PATH, 0);
		}
	}else{ //load path setting
	// --- ^^^ ---  '//hogehoge' shortcut convert (may be etcfunc...)
		Path_Remove_Backslash(buf); // '\\ \\ \\ ...' Backslash heaven guard.
		Path_Add_Backslash(buf);
		set_destdir(buf, MAX_PATH, 0);
	}

	//opendir (Open Folder when meltice unpacked archives.), default=true.
	GetPrivateProfileString(cUnPackSection,"OpenDir","1",buf,2,inipath);
	if(*buf=='0')	set_opendir( false, 0);	// Opendir=0 -> false
	else 			set_opendir(  true, 0);	// OpenDir=1 -> true

	//opdirex (customize for open folder when unpacked archives.)
	GetPrivateProfileString(cUnPackSection,"OpDirEx","Explorer",buf,MAX_PATH,inipath);
	set_opdirex(buf, MAX_PATH, 0);

	//samedir (meltice unpack archives to samedir.), default=false.
	GetPrivateProfileString(cUnPackSection,"SameDir","0",buf,2,inipath);
	if(*buf=='0')	set_samedir( false, 0);	// Samedir=0 -> false
	else 			set_samedir(  true, 0);	// SameDir=1 -> true

	//optmdir (Output directory optimized.), default=true.
	//  ... This setting for unpack only!
	GetPrivateProfileString(cUnPackSection,"OptmDir","1",buf,2,inipath);
	if(*buf=='0')	{	set_optmdir( false, 0);set_ncutdir( false, 0);	}// Optmdir=0 -> false
	else 			{	set_optmdir(  true, 0);set_ncutdir(  true, 0);	}// OptmDir=1 -> true

	//noardll (Not use common archiver prj's dll), default=false.
	//for debug option, Hidden options.
	GetPrivateProfileString(cUnPackSection,"NoArDll","0",buf,2,inipath);
	if(*buf=='0')	set_noardll( false, 0);	// NoArDll=0 -> false
	else 			set_noardll(  true, 0);	// NoArDll=1 -> true

	//PackExp, (ex. PackExp=Lzh,Zip,Cab,Tgz,Yz1 )
	GetPrivateProfileString(cUnPackSection,"PackExp","Lzh,Lzs,Lha,Zip,Jar,Arj,Cab,Tar,Tgz,Gz,Tbz,Bz2,Bza,Gza,Yz1,Gca,Ace,Rar,Exe,Jak,Mxd",buf,MAX_PATH,inipath);
	set_packexp(buf);

// [Pack] Section.
	GetPrivateProfileString(cPackSection,"PackMxd","Frostice.mxd",buf,MAX_PATH,inipath);
	set_packmxd(buf);

	delete [] desktopdir;
	delete [] buf;
	delete [] inipath;
}

// save settings to ini file.
void MemcfgToInicfg(HWND hDlg)
{
	//INI path setting...
	char *inipath = new char[MAX_PATH]; // INI file
	char *pathbuf = new char[MAX_PATH]; // DestDir / DirOpEx Path.

	char *tmppath = new char[MAX_PATH]; // Temp Path
	char *dskpath = new char[MAX_PATH]; // Temp Path
	char *sp_path = new char[MAX_PATH]; // Temp Path

	GetTempPath(MAX_PATH,tmppath);
	GetLongPathName95(tmppath,tmppath); //for LFN TempPath on Windows 2000 and XP.
	Path_Add_Backslash(tmppath);
	GetWinSpecialFolderPath(dskpath, CSIDL_DESKTOP);
	Path_Add_Backslash(dskpath);


	GetINIfilename(inipath,MAX_PATH);


// [Global] Section.
	//AutoLanguage
	if( get_autolang() == true)
		WritePrivateProfileString(cGlobalSection, "AutoLang", "1", inipath);
	else 
		WritePrivateProfileString(cGlobalSection, "AutoLang", "0", inipath);

    RECT rc;
    GetWindowRect(hDlg, (LPRECT)&rc); //Get now window position.
    wsprintf(pathbuf,"%d",rc.left); //pathbuf = not path, buffer :-P
	WritePrivateProfileString(cGlobalSection, "Dialog_X", pathbuf, inipath);
    wsprintf(pathbuf,"%d",rc.top);
	WritePrivateProfileString(cGlobalSection, "Dialog_Y", pathbuf, inipath);

	//WriteLog
	if( get_writelog() == true)
		WritePrivateProfileString(cGlobalSection, "WriteLog", "1", inipath);
	else 
		WritePrivateProfileString(cGlobalSection, "WriteLog", "0", inipath);


// [UnPack] Section.
	//meltdir save...
	get_destdir(pathbuf, MAX_PATH, 0); //get meltdir.
	Path_Add_Backslash(pathbuf);

	if(lstrcmp(dskpath, pathbuf)==0)
	{
		WritePrivateProfileString(cUnPackSection,"DestDir","//desktop",inipath);
	}else if(lstrcmp(tmppath, pathbuf)==0)
	{ 
		WritePrivateProfileString(cUnPackSection,"DestDir","//temp",inipath);
	}else if(
		lstrcmp(GetWinSpecialFolderPath(sp_path, CSIDL_PERSONAL), pathbuf)==0)
	{ 
		WritePrivateProfileString(cUnPackSection,"DestDir","//mydoc",inipath);
	}else if(lstrcmp(GetExecDir(tmppath,MAX_PATH), pathbuf)==0)
	{ 
		WritePrivateProfileString(cUnPackSection,"DestDir","//execdir",inipath);
	}else
	{
		WritePrivateProfileString(cUnPackSection,"DestDir",pathbuf,inipath);
	}

	//opendir save...
	if( get_opendir(0) == true)
		WritePrivateProfileString(cUnPackSection,"OpenDir","1",inipath);
	else 
		WritePrivateProfileString(cUnPackSection,"OpenDir","0",inipath);


	//opdirex save...
	get_opdirex(pathbuf, MAX_PATH,0);
	WritePrivateProfileString(cUnPackSection,"OpDirEx",pathbuf,inipath);

	//samedir save...
	if( get_samedir(0) == true)
		WritePrivateProfileString(cUnPackSection,"SameDir","1",inipath);
	else 
		WritePrivateProfileString(cUnPackSection,"SameDir","0",inipath);


	//optmdir save...
	if( get_optmdir(0) == true)
		WritePrivateProfileString(cUnPackSection,"OptmDir","1",inipath);
	else 
		WritePrivateProfileString(cUnPackSection,"OptmDir","0",inipath);


	//PackExp, (ex. PackExp=Lzh,Zip,Cab,Tgz,Yz1 )
	WritePrivateProfileString(cUnPackSection,"PackExp",get_packexp(tmppath),inipath);

// [Pack] Section.
	//PackMxd. (Default:Frostice.mxd)
/* ** Temp Comment out.
	WritePrivateProfileString(cPackSection,"PackMxd",get_packmxd(tmppath),inipath);
*/
	delete [] tmppath;
	delete [] dskpath;
	delete [] sp_path;

	delete [] pathbuf;
	delete [] inipath;
	
}
